==> mutual_credit-5.0.11.tar.gz: mutual_credit/src/Element/WorthsForm.php <==
<?php

namespace Drupal\mcapi\Element;

use Drupal\mcapi\Entity\Currency;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Component\Utility\Html as HtmlUtility;

/**
 * Provides a worth field form element.
 *
 * @FormElement("worths_form")
 */
class WorthsForm extends FormElement {
  use \Drupal\Core\Render\Element\CompositeFormElementTrait;

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#tree' => TRUE,
      '#process' => [
        [$class, 'processCurrencies'],
        [$class, 'process'],
      ],
      '#element_validate' => [
        [$class, 'validate'],
      ],
      '#pre_render' => [
        [$class, 'preRenderCompositeFormElement'],
      ],
      '#attributes' => ['class' => ['worth-element']],
      '#config' => FALSE,
      '#allowed_curr_ids' => array_keys(Currency::loadMultiple()),
    ];
  }

  /**
   * Ensure the worth field is showing the right subset of currencies.
   *
   * @note The #default_value is then used to build the widget.
   */
  public static function processCurrencies($element, FormStateInterface $form_state, $form) {
    // Populate #allowed_curr_ids from the default value OR all currencies.
    if (empty($element['#default_value']) or empty($element['#default_value'][0])) {
      $element['#default_value'] = [];
      foreach ($element['#allowed_curr_ids'] as $curr_id) {
        $element['#default_value'][] = [
          'curr_id' => $curr_id,
          'value' => '',
        ];
      }
    }
    else {
//      foreach (Self::currMap($element['#default_value']) as $curr_id => $delta) {
//        $element['#allowed_curr_ids'] = $all_curr_ids;
//      }
    }
    if ($element['#config']) {
      foreach ($element['#default_value'] as &$worth) {
        if ($worth['value'] === '') {
          $worth['value'] = '0';
        }
      }
    }
    return $element;
  }

  /**
   * Form element processor callback.
   *
   * Make widget(s) according to each currency format of each #default_value.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    // expected by CompositeFormElementTrait.
    // Help on RenderElement says this should be set automatically for form elements
    $element['#id'] = '#'. $form_state->get('form_id') .'-worth';
    $map = Self::currMap($element['#default_value']);
    foreach ($element['#allowed_curr_ids'] as $delta => $curr_id) {
      $item = (isset($map[$curr_id]) and isset($element['#default_value'][$map[$curr_id]])) ?
        $element['#default_value'][$map[$curr_id]] :
        ['curr_id' => $curr_id, 'value' => 0];
      $element[$delta] = [
        '#type' => 'worth_form',
        '#default_value' => $item,
        '#allowed_curr_ids' => [$curr_id],
        '#config' => $element['#config'],
        '#oneofmany' => count($element['#default_value']) > 1,
        '#theme_wrappers' => [],
        '#id' => HtmlUtility::getUniqueId('edit-' . implode('-', array_merge($element['#parents'], [$delta]))),
      ];
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @todo Stop the values of the child elements overwriting the $output here?
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input === FALSE) {
      return $element['#default_value'];
    }
    $output = [];
    if ($input === NULL) {
      return $output;
    }
    foreach ($input as $delta => $item) {
      $output[] = WorthForm::worthValue(
        $item,
        $element['#config'],
        $element['#default_value']
      );
    }
    // @todo it is this which is checked using WorthFieldList::filterEmptyItems
    return $output;
  }

  /**
   * Validate callback.
   *
   * All validation takes place in the children.
   */
  public static function validate(&$element, FormStateInterface $form_state) {
    if (is_array($element['#value'])) {
      foreach ($element['#value'] as $key => $val) {
        if (!isset($val['value'])) {
          unset($element['#value'][$key]);
        }
      }
      $element['#value'] = array_values($element['#value']);
      $form_state->setValue($element['#name'], $element['#value']);
    }
  }

  /**
   * Helper.
   *
   * Map the currencies to worth deltas.
   *
   * @param array $value
   *   A value from a worth field, containing curr_id and value.
   *
   * @return array
   *   The deltas, keyed by curr_id.
   */
  private static function currMap($value) {
    if (empty($value)) {
      return array_keys(Currency::loadMultiple());
    }
    $map = [];
    foreach ($value as $key => $item) {
      $map[$item['curr_id']] = $key;
    }
    return $map;
  }

}

==> mutual_credit-5.1.0.tar.gz: mutual_credit/src/Element/WorthsForm.php <==
<?php

namespace Drupal\mcapi\Element;

use Drupal\mcapi\Entity\Currency;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Component\Utility\Html as HtmlUtility;

/**
 * Provides a worth field form element.
 *
 * @FormElement("worths_form")
 */
class WorthsForm extends FormElement {
  use \Drupal\Core\Render\Element\CompositeFormElementTrait;

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#tree' => TRUE,
      '#process' => [
        [$class, 'processCurrencies'],
        [$class, 'process'],
      ],
      '#element_validate' => [
        [$class, 'validate'],
      ],
      '#pre_render' => [
        [$class, 'preRenderCompositeFormElement'],
      ],
      '#attributes' => ['class' => ['worth-element']],
      '#config' => FALSE,
      '#minus' => FALSE,
      '#allowed_curr_ids' => array_keys(Currency::loadMultiple())
    ];
  }

  /**
   * Ensure the worth field is showing the right subset of currencies.
   *
   * @note The #default_value is then used to build the widget.
   */
  public static function processCurrencies($element, FormStateInterface $form_state, $form) {
    // Populate #allowed_curr_ids from the default value OR all currencies.
    if (empty($element['#default_value']) or empty($element['#default_value'][0])) {
      $element['#default_value'] = [];
      foreach ($element['#allowed_curr_ids'] as $curr_id) {
        $element['#default_value'][] = [
          'curr_id' => $curr_id,
          'value' => '',
        ];
      }
    }
    else {
//      foreach (Self::currMap($element['#default_value']) as $curr_id => $delta) {
//        $element['#allowed_curr_ids'] = $all_curr_ids;
//      }
    }
    if ($element['#config']) {
      foreach ($element['#default_value'] as &$worth) {
        if ($worth['value'] === '') {
          $worth['value'] = '0';
        }
      }
    }
    return $element;
  }

  /**
   * Form element processor callback.
   *
   * Make widget(s) according to each currency format of each #default_value.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    // expected by CompositeFormElementTrait.
    // Help on RenderElement says this should be set automatically for form elements
    $element['#id'] = '#'. $form_state->get('form_id') .'-worth';
    $map = Self::currMap($element['#default_value']);
    foreach ($element['#allowed_curr_ids'] as $delta => $curr_id) {
      $item = (isset($map[$curr_id]) and isset($element['#default_value'][$map[$curr_id]])) ?
        $element['#default_value'][$map[$curr_id]] :
        ['curr_id' => $curr_id, 'value' => 0];
      $element[$delta] = [
        '#type' => 'worth_form',
        '#default_value' => $item,
        '#allowed_curr_ids' => [$curr_id],
        '#config' => $element['#config'],
        '#oneofmany' => count($element['#default_value']) > 1,
        '#theme_wrappers' => [],
        '#id' => HtmlUtility::getUniqueId('edit-' . implode('-', array_merge($element['#parents'], [$delta]))),
      ];
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @todo Stop the values of the child elements overwriting the $output here?
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input === FALSE) {
      return $element['#default_value'];
    }
    $output = [];
    if ($input === NULL) {
      return $output;
    }
    foreach ($input as $delta => $item) {
      $output[] = WorthForm::worthValue(
        $item,
        $element['#config'],
        (bool)$element['#minus']
      );
    }
    // @todo it is this which is checked using WorthFieldList::filterEmptyItems
    return $output;
  }

  /**
   * Validate callback.
   *
   * All validation takes place in the children.
   */
  public static function validate(&$element, FormStateInterface $form_state) {
    if (is_array($element['#value'])) {
      foreach ($element['#value'] as $key => $val) {
        if (!isset($val['value'])) {
          unset($element['#value'][$key]);
        }
      }
      $element['#value'] = array_values($element['#value']);
      $form_state->setValue($element['#name'], $element['#value']);
    }
  }

  /**
   * Helper.
   *
   * Map the currencies to worth deltas.
   *
   * @param array $value
   *   A value from a worth field, containing curr_id and value.
   *
   * @return array
   *   The deltas, keyed by curr_id.
   */
  private static function currMap($value) {
    if (empty($value)) {
      return array_keys(Currency::loadMultiple());
    }
    $map = [];
    foreach ($value as $key => $item) {
      $map[$item['curr_id']] = $key;
    }
    return $map;
  }

}
